// -------------------------------------------------------------------------
// Copyright @ 1997 TCK Software, Incorporated
// All Rights Reserved
// -------------------------------------------------------------------------
// VTreeView.cpp : implementation of the CVTreeView class
//

#include "stdafx.h"
#include "vfTest.h"

#include "vfTestDoc.h"
#include "VTreeView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDC_VFORM	100
#define IDC_VFORM2	101

/////////////////////////////////////////////////////////////////////////////
// CVTreeView

IMPLEMENT_DYNCREATE(CVTreeView, CView)

BEGIN_MESSAGE_MAP(CVTreeView, CView)
	//{{AFX_MSG_MAP(CVTreeView)
	ON_WM_CREATE()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	ON_COMMAND(ID_FILE_PRINT_SCREEN, OnFilePrintScreen)
	ON_COMMAND(ID_FILE_PRINT_ALL, OnFilePrintAll)
	ON_COMMAND(ID_FILE_PRINT_SELECTED, OnFilePrintSelected)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVTreeView construction/destruction

CVTreeView::CVTreeView()
{
	// TODO: add construction code here

}

CVTreeView::~CVTreeView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CVTreeView drawing

void CVTreeView::OnDraw(CDC* pDC)
{
	CVfTestDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}

/////////////////////////////////////////////////////////////////////////////
// CVTreeView printing

void CVTreeView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	m_vForm.DoPrint(pDC, pInfo);	
	// CView::OnPrint(pDC, pInfo);
}

BOOL CVTreeView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return m_vForm.DoPreparePrinting(this, pInfo);
	// return DoPreparePrinting(pInfo);
}

void CVTreeView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// Set the page info again, since the user may have changed the printer since
	// we set it up in OnPreparePrinting
	m_vForm.CalcPageInfo(pDC, pInfo);
}

void CVTreeView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CVTreeView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	CView::OnPrepareDC(pDC, pInfo);
	m_vForm.DoPrepareDC(pDC, pInfo);
}

void CVTreeView::OnFilePrintScreen() 
{
	m_vForm.PrintStyle(VFPRINT_SCREEN);
	CView::OnFilePrintPreview();
}

void CVTreeView::OnFilePrintAll() 
{
	m_vForm.PrintStyle(VFPRINT_ALL);
	CView::OnFilePrintPreview();
}

void CVTreeView::OnFilePrintSelected() 
{
	m_vForm.PrintStyle(VFPRINT_SELECTED);
	CView::OnFilePrintPreview();	
}


/////////////////////////////////////////////////////////////////////////////
// CVTreeView diagnostics

#ifdef _DEBUG
void CVTreeView::AssertValid() const
{
	CView::AssertValid();
}

void CVTreeView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CVfTestDoc* CVTreeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVfTestDoc)));
	return (CVfTestDoc*)m_pDocument;
}
#endif //_DEBUG

void CVTreeView::ResetPalette()
{
	m_palMsgHandler.DoRealizePalette(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// CVTreeView message handlers

int CVTreeView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	int rc;
	CRect rect(10, 10, 400, 400);

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_vForm.SetPalMH(&m_palMsgHandler);
	rc = m_vForm.Create(_T("VForm"), rect, this, IDC_VFORM);
	if(!rc)
		MessageBox(_T("Could not create VForm"), _T("Error")); 

/*  ---- Example of creating 2 VForms in same window ---
	m_vForm.Register();
	rc = m_vForm.CreateEx(WS_EX_CLIENTEDGE,
			m_vForm.ClassName(), _T("VForm"),
            WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | WS_TABSTOP | WS_VSCROLL,
            10, 10, 400, 400,
			this->GetSafeHwnd(),(HMENU)IDC_VFORM);


	rc = m_vForm2.CreateEx(WS_EX_CLIENTEDGE,
			m_vForm.ClassName(), _T("VForm2"),
            WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | WS_TABSTOP | WS_VSCROLL,
            415, 10, 400, 340,
			this->GetSafeHwnd(),(HMENU)IDC_VFORM2);
	if(!rc)
		MessageBox(_T("Could not create form 2"), _T("Error")); 
*/
	return 0;
}


// -------------------------------------------------------------------------
// OnEraseBkgnd - overridden to draw gray background where VForm does not fit
//  on the bottom -> VForm will always be a row heigh multiple + header and
//  footer height.  If the window height is not on one of these multiples, a
//  small area will be displayed below the footer.  
//  This just makes that area gray.
// -------------------------------------------------------------------------
BOOL CVTreeView::OnEraseBkgnd(CDC* pDC) 
{
/*	// --- Only use if using snap resize 
	// --- this will cause flicker when sizing
	CBrush backBrush(GetSysColor(COLOR_3DFACE));
    CBrush* pOldBrush = pDC->SelectObject(&backBrush);
 
    CRect rect;
    pDC->GetClipBox(&rect);     // Erase the area needed
    pDC->PatBlt(rect.left, rect.top, rect.Width(), rect.Height(), PATCOPY);
    pDC->SelectObject(pOldBrush);
*/
    return TRUE;	
//	return CView::OnEraseBkgnd(pDC);
}

void CVTreeView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	m_vForm.ResizeToParent();
}

void CVTreeView::OnSetFocus(CWnd* pOldWnd) 
{
	// CView::OnSetFocus(pOldWnd);
	m_vForm.SetFocus();
}

void CVTreeView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	VBitmap* pBmp = m_vForm.GetBmp();
	if (pBmp) 
	{
		if (!m_palMsgHandler.IsHooked())
			m_palMsgHandler.Install(this, pBmp->GetPalette());

		// The following line is required because MFC does not send
		// WM_INITIALUPDATE through normal channels. Only realize in
		// foreground if I have the focus (could be updating all views
		// from OnFontChange)
		m_palMsgHandler.DoRealizePalette(m_hWnd==::GetFocus());
	}	
}
